import { Result } from 'ts-res';
import { ConfigurationDataType, RuleType } from '../clientConfiguration';
import { KameleoonError } from '../kameleoonError';
import { KameleoonData } from '../kameleoonData';
import { TrackingStatus } from '../types';
import { TargetingDataType } from '../targeting';
export type ClientDataType = {
    data: ConfigurationDataType;
    lastUpdate: string;
};
export type DataInfoType = {
    nextVisitTime: number;
    nextDataCleanup?: number;
    unsentData: {
        [visitorCode: string]: KameleoonData[];
    };
    mappingIdentifiers?: {
        [visitorCode: string]: MappingIdentifierType;
    };
};
export type MappingIdentifierType = {
    mappingIdentifier: string;
    timestamp: number;
};
export type FeatureFlagVariationType = {
    variationId: number;
    isTargetedRule: boolean;
    status: TrackingStatus;
};
export type FeatureFlagVariationsType = {
    [experimentId: string]: FeatureFlagVariationType;
};
export type VariationDataType = {
    [visitorCode: string]: FeatureFlagVariationsType;
};
export type ForcedFeatureVariationType = {
    featureKey: string;
    rule: ForcedVariationRuleType | null;
    varByExp: ForcedVariationVarByExpType | null;
    simulated: boolean;
};
export type ForcedExperimentVariationType = {
    rule: ForcedVariationRuleType;
    varByExp: ForcedVariationVarByExpType;
    forceTargeting: boolean;
};
export type ForcedVariationRuleType = {
    type: RuleType;
    experimentId: number;
};
export type ForcedVariationVarByExpType = {
    variationKey: string;
    variationId: number;
};
export type SimulatedVariationType = {
    featureKey: string;
    variationId: number | null;
    experimentId: number;
};
export type ForcedExperimentVariationDataType = {
    [visitorCode: string]: {
        [experimentId: number]: ForcedExperimentVariationType;
    };
};
export type ForcedFeatureVariationDataType = {
    [simulated: number]: {
        [visitorCode: string]: {
            [featureKey: string]: ForcedFeatureVariationType;
        };
    };
};
export type ForcedVariationDataType = {
    [visitorCode: string]: {
        [featureKey: string]: ForcedFeatureVariationType;
    };
};
export type ConsentDataType = {
    [visitorCode: string]: boolean | {
        consent: boolean;
    };
};
export type ScheduledVisitorsType = {
    [visitorCode: string]: {
        isConsentProvided: boolean;
    };
};
export type TrackingDataType = {
    scheduledVisitors: ScheduledVisitorsType;
};
/**
 * @type `StorageDataType` - an enumeration type which represents a data stored on SDK storage
 *
 * `string` represents visitor code storage with only one string value
 * */
export type StorageDataType = ClientDataType | TargetingDataType | TrackingDataType | VariationDataType | ConsentDataType | DataInfoType | string;
/**
 * @interface an interface of External Storage which must be implemented to be used as external storage an SDK
 * */
export interface IExternalStorage<T = StorageDataType> {
    /**
     * @method read - reads data from the storage
     * @param {string} key - a key, which different SDK data blocks are stored under
     * @returns {T | null} representation of generic Kameleoon data type `T` or `null` if the storage was read successfully but it's empty
     * */
    read: (key: string) => T | null;
    /**
     * @method write - writes data to storage
     * @param {string} key - a key, which different SDK data blocks are stored under
     * @param {T} data - representation of generic Kameleoon data type
     * @returns {void}
     * */
    write: (key: string, data: T) => void;
}
/**
 * @interface an interface which serves as a wrapper over `IExternalStorage` for internal SDK usage
 * */
export interface IStorage<T> {
    read: () => Result<T, KameleoonError>;
    write: (data: T) => Result<void, KameleoonError>;
    update: (data: Partial<T>) => Result<void, KameleoonError>;
}
